@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 # WURFL 2 Python - Wireless Universal Resource File to Python conversion utility
-# Copyright (C) 2006 Armand Lynch
+# Copyright (C) 2004-2008 Armand Lynch
 #
 # This library is free software; you can redistribute it and/or modify it
 # under the terms of the GNU Lesser General Public License as published by the
@@ -33,7 +33,7 @@ from pywurfl.wurflprocessor import DeviceHandler, WurflProcessor, op, main
 
 __version__ = "4.3.0a"
 __license__ = "LGPL"
-__copyright__ = "Copyright 2004-6, Armand Lynch"
+__copyright__ = "Copyright 2004-2008, Armand Lynch"
 __author__ = "Armand Lynch <lyncha@users.sourceforge.net>"
 __contributors__ = "Pau Aliagas <pau@newtral.org>"
 __url__ = "http://celljam.net/"
@@ -45,6 +45,7 @@ class DeviceSerializer(DeviceHandler):
     idstr = "devices.devids['''%s'''] = "
     new_caps = idstr + "devclass(%s, '''%s''', '''%s''', %s, {%s})\n"
     no_caps = idstr + "devclass(%s, '''%s''', '''%s''', %s, None)\n"
+    capability_types = {}
 
     def __str__(self):
         s = []
@@ -57,25 +58,26 @@ class DeviceSerializer(DeviceHandler):
 
         for cap in sorted(self.capabilities):
             value = self.capabilities[cap]
-            try:
-                c.append("'''%s''':%d" % (cap, int(value)))
-            except ValueError:
+            cap_type = self.capability_types.get(cap, None)
+            if cap_type == int:
+                c.append("'''%s''':%d" % (cap, int(value.strip())))
+            elif cap_type == float:
+                c.append("'''%s''':%f" % (cap, float(value.strip())))
+            elif cap_type == bool:
                 if value.lower() == "true":
                     c.append("'''%s''':True" % cap)
                 elif value.lower() == "false":
                     c.append("'''%s''':False" % cap)
-                else:
-                    c.append("'''%s''':'''%s'''" % (cap, value))
+            else:
+                c.append("'''%s''':'''%s'''" % (cap, value))
 
         if c:
-            s.append(DeviceSerializer.new_caps % (self.devid, parent,
-                                                  self.devid, self.devua,
-                                                  self.actual_device_root,
-                                                  ','.join(c)))
+            s.append(self.new_caps % (self.devid, parent, self.devid,
+                                      self.devua, self.actual_device_root,
+                                      ','.join(c)))
         else:
-            s.append(DeviceSerializer.no_caps % (self.devid, parent,
-                                                 self.devid, self.devua,
-                                                 self.actual_device_root))
+            s.append(self.no_caps % (self.devid, parent, self.devid, self.devua,
+                                     self.actual_device_root))
 
         return ''.join(s)
 
@@ -110,6 +112,8 @@ class WurflPythonWriter(WurflProcessor):
                 self.groups[group.get('id')].append(attrs["name"])
 
     def start_process(self):
+        self.device_handler.capability_types = get_normalized_types(self.tree)
+
         version = self.tree.findtext("*/ver").strip()
         self.outfile.write('"""\n%s\n\n' % self.outfile.name)
         self.outfile.write("Generated on: %s\n" % ctime())
@@ -130,7 +134,53 @@ class WurflPythonWriter(WurflProcessor):
         self.outfile.write("    devices.devuas[device.devua] = device\n")
         self.outfile.write("del device\n")
         self.outfile.write("\n")
-        self.outfile.write("devices._normalize_types()\n")
+
+
+def get_normalized_types(tree):
+    caps = {}
+    for capability in tree.findall("devices/device/group/capability"):
+        name = capability.attrib['name']
+        value = capability.attrib['value']
+        if name not in caps:
+            try:
+                int(value)
+                caps[name] = int
+                continue
+            except (TypeError, ValueError):
+                pass
+            try:
+                float(value)
+                caps[name] = float
+                continue
+            except (TypeError, ValueError):
+                pass
+
+            if value.strip().lower() in ("true", "false"):
+                caps[name] = bool
+                continue
+            else:
+                caps[name] = str
+        else:
+            if caps[name] == str:
+                continue
+            elif caps[name] == bool:
+                if value.strip().lower() in ("true", "false"):
+                    continue
+                else:
+                    caps[name] = str
+            elif caps[name] == float:
+                try:
+                    float(value)
+                    continue
+                except (TypeError, ValueError):
+                    caps[name] = str
+            elif caps[name] == int:
+                try:
+                    int(value)
+                    continue
+                except (TypeError, ValueError):
+                    caps[name] = str
+    return caps
 
 
 if __name__ == "__main__":
