@@ -1,5 +1,5 @@
 # pywurfl - Wireless Universal Resource File Tools in Python
-# Copyright (C) 2006 Armand Lynch
+# Copyright (C) 2006-2008 Armand Lynch
 #
 # This library is free software; you can redistribute it and/or modify it
 # under the terms of the GNU Lesser General Public License as published by the
@@ -23,18 +23,19 @@ pywurfl - Python tools for processing and querying the Wireless Universal Resour
 """
 
 import re
+import md5
 from copy import copy
 
-from pywurfl.exceptions import (BaseException, ActualDeviceRootNotFound,
+from pywurfl.exceptions import (WURFLException, ActualDeviceRootNotFound,
                                 DeviceNotFound, ExistsException)
 
 
 __author__ = "Armand Lynch <lyncha@users.sourceforge.net>"
 __contributors__ = "Pau Aliagas <pau@newtral.org>"
-__copyright__ = "Copyright 2006, Armand Lynch"
+__copyright__ = "Copyright 2006-2008, Armand Lynch"
 __license__ = "LGPL"
 __url__ = "http://celljam.net/"
-__version__ = "6.3.1b"
+__version__ = "6.4.0b"
 __all__ = ['devclass', 'Devices']
 
 
@@ -238,7 +239,7 @@ class Devices(object):
         @type group: string
         """
         if group not in self.devids['generic'].groups:
-            raise BaseException("'%s' group not found" % group)
+            raise WURFLException("'%s' group not found" % group)
         caps = self.devids['generic'].groups[group]
         generic = self.devids['generic']
         for cap in caps:
@@ -295,7 +296,7 @@ class Devices(object):
             if capability in self.devids['generic'].groups[group]:
                 break
         else:
-            raise BaseException("'%s' capability not found" % capability)
+            raise WURFLException("'%s' capability not found" % capability)
         generic = self.devids['generic']
         self._remove_capability(generic, capability)
         self.devids['generic'].groups[group].remove(capability)
@@ -351,7 +352,7 @@ class Devices(object):
         @type capabilities: dict
         """
         if child == 'generic':
-            raise BaseException("cannot insert device before generic device")
+            raise WURFLException("cannot insert device before generic device")
         if child not in self.devids:
             raise DeviceNotFound(child)
         if devid in self.devids:
@@ -424,7 +425,7 @@ class Devices(object):
         if devid not in self.devids:
             raise DeviceNotFound(devid)
         if devid == 'generic':
-            raise BaseException("cannot remove generic device")
+            raise WURFLException("cannot remove generic device")
 
         device = self.devids[devid]
         parent_device = device.__bases__[0]
@@ -448,7 +449,7 @@ class Devices(object):
         if devid not in self.devids:
             raise DeviceNotFound(devid)
         if devid == 'generic':
-            raise BaseException("cannot remove generic device")
+            raise WURFLException("cannot remove generic device")
 
         device = self.devids[devid]
         self._remove_tree(devid)
@@ -500,6 +501,15 @@ class Devices(object):
         """
         return self.devuas.iterkeys()
 
+    @property
+    def md5_hexdigest(self):
+        """
+        Return MD5 hex digest for all WURFL data
+        """
+        data = []
+        [data.append(str(self.devids[x]())) for x in sorted(self.devids)]
+        return md5.new(''.join(data)).hexdigest()
+
     def __iter__(self):
         return self.devids.__iter__()
 
@@ -565,5 +575,5 @@ class Devices(object):
         if not self._name_test_re.match(value):
             msg = "%s '%s' does not conform to regexp "
             msg += "r'^(_|[a-z])(_|[a-z]|[0-9])+$'"
-            raise BaseException(msg % (name, value))
+            raise WURFLException(msg % (name, value))
 
