@@ -1,5 +1,5 @@
 # pywurfl Algorithms - Wireless Universal Resource File UA search algorithms
-# Copyright (C) 2006 Armand Lynch
+# Copyright (C) 2006-2008 Armand Lynch
 #
 # This library is free software; you can redistribute it and/or modify it
 # under the terms of the GNU Lesser General Public License as published by the
@@ -28,7 +28,7 @@ from pywurfl.exceptions import DeviceNotFound
 
 
 __author__ = "Armand Lynch <lyncha@users.sourceforge.net>"
-__copyright__ = "Copyright 2006, Armand Lynch"
+__copyright__ = "Copyright 2006-2008, Armand Lynch"
 __license__ = "LGPL"
 __url__ = "http://celljam.net/"
 
@@ -117,7 +117,8 @@ class Tokenizer(Algorithm):
     """
     Tokenizer Search Algorithm
     """
-    tokenize_chars = ('/', '.', '-', '_', ' ')
+    tokenize_chars = ('/', '.', ',', ';', '-', '_', ' ', '(', ')')
+    base_regex = '[\\'+'\\'.join(tokenize_chars)+']*'
 
     def __init__(self, devwindow=30):
         """
@@ -149,7 +150,7 @@ class Tokenizer(Algorithm):
         regex = ''
         for t in tokens:
             if regex:
-                regex += '[\/\.\-_ ]*' + t
+                regex += self.base_regex + t
             else:
                 regex += t
 
