@@ -1,5 +1,5 @@
 # pywurfl Exceptions
-# Copyright (C) 2006 Armand Lynch
+# Copyright (C) 2006-2008 Armand Lynch
 #
 # This library is free software; you can redistribute it and/or modify it
 # under the terms of the GNU Lesser General Public License as published by the
@@ -23,19 +23,19 @@ Exception Classes
 """
 
 __author__ = "Armand Lynch <lyncha@users.sourceforge.net>"
-__copyright__ = "Copyright 2006, Armand Lynch"
+__copyright__ = "Copyright 2006-2008, Armand Lynch"
 __license__ = "LGPL"
 __url__ = "http://celljam.net/"
 
 
-class BaseException(Exception):
+class WURFLException(Exception):
     """
     pywurfl base exception class.
     """
     pass
 
 
-class ExistsException(BaseException):
+class ExistsException(WURFLException):
     """
     General exception class
 
@@ -43,7 +43,7 @@ class ExistsException(BaseException):
     """
 
 
-class DeviceNotFound(BaseException):
+class DeviceNotFound(WURFLException):
     """
     Device Not Found exception class
 
@@ -53,7 +53,7 @@ class DeviceNotFound(BaseException):
     pass
 
 
-class ActualDeviceRootNotFound(BaseException):
+class ActualDeviceRootNotFound(WURFLException):
     """
     Actual Device Root Not Found exception class
 
