@@ -1,5 +1,5 @@
 # pywurfl QL - Wireless Universal Resource File Query Language in Python
-# Copyright (C) 2006 Armand Lynch
+# Copyright (C) 2006-2008 Armand Lynch
 #
 # This library is free software; you can redistribute it and/or modify it
 # under the terms of the GNU Lesser General Public License as published by the
@@ -140,50 +140,137 @@ query := select_statement where_statement
 
 select_statement := 'select' ('device' | 'id' | 'ua')
 
-where_statement := 'where where_test (boolop where_test)*
+where_statement := 'where' + where_expression
 
-where_test := (any_test | all_test | capability_test)
+where_expression := where_test (boolop where_test)*
 
-any_test := 'any' capability_list operator value
+where_test := (any_statement | all_statement | expr_test)
 
-all_test := 'all' capability_list operator value
+any_statement := 'any' '(' expr_list ')' operator expr
+
+all_statement := 'all' '(' expr_list ')' operator expr
 
 capability := alphanums ('_' alphanums)*
 
-capability_list := '(' capability, capability (',' capability)* ')'
+expr_test := expr operator expr
 
-capability_test := capability operator value
+expr_list := expr (',' expr)*
+
+expr := types attributes_methods_concat | capability attributes_methods_concat
+
+attributes_methods_concat := ('.' method '(' method_args? ')')*
+
+method_args := (method_arg (',' method_arg)*)
+
+method_arg := (types | expr)
+
+method := ('_' alphanums)*
 
 operator := ('='|'!='|'<'|'>'|'>='|'<=')
 
-value := (<quote> string <quote> | integer | boolean)
+types := (<quote> string <quote> | integer | boolean)
 
 boolean := ('true' | 'false')
 
 boolop := ('and' | 'or')
 """
 
+import re
 import operator
 
 from pyparsing import (CaselessKeyword, Forward, Group, ParseException,
                        QuotedString, StringEnd, Suppress, Word, ZeroOrMore,
-                       alphanums, alphas, nums, oneOf)
+                       alphanums, alphas, nums, oneOf, delimitedList)
 
-from pywurfl.exceptions import BaseException
+from pywurfl.exceptions import WURFLException
 
 
 __author__ = "Armand Lynch <lyncha@users.sourceforge.net>"
-__copyright__ = "Copyright 2006, Armand Lynch"
+__contributors__ = "Gabriele Fantini <gabriele.fantini@staff.dada.net>"
+__copyright__ = "Copyright 2006-2008, Armand Lynch"
 __license__ = "LGPL"
 __url__ = "http://celljam.net/"
 __all__ = ['QueryLanguageError', 'QL']
 
 
-class QueryLanguageError(BaseException):
+class QueryLanguageError(WURFLException):
     """Base exception class for pywurfl.ql"""
     pass
 
 
+def _toNum(s, l, toks):
+    """Convert to pywurfl number type"""
+    n = toks[0]
+    try:
+        return TypeNum(int(n))
+    except ValueError, e:
+        return TypeNum(float(n))
+
+
+def _toBool(s, l, toks):
+    """Convert to pywurfl boolean type"""
+    val = toks[0]
+    if val.lower() == 'true':
+        return TypeBool(True)
+    elif val.lower() == 'false':
+        return TypeBool(False)
+    else:
+        raise QueryLanguageError("Invalid boolean value '%s'" % val)
+
+
+def _toStr(s, l, toks):
+    """Convert to pywurfl string type"""
+    val = toks[0]
+    return TypeStr(val)
+
+
+class _Type:
+    def __init__(self, py_value):
+        self.py_value = py_value
+
+    def __getattr__(self, method):
+        return getattr(self.py_value, method)
+
+
+class TypeNone(_Type):
+    pass
+
+
+class TypeNum(_Type):
+    pass
+
+
+class TypeStr(_Type):
+    def substr(self, begin, end):
+        try:
+            return self.py_value[begin:end]
+        except IndexError, e:
+            return None
+
+    def _match(self, regex, num=0, flags=0):
+        if re.compile(regex, flags).match(self.py_value, num) is None:
+            return False
+        else:
+            return True
+
+    def match(self, regex, num=0):
+        return self._match(regex, num)
+
+    def imatch(self, regex, num=0):
+        return self._match(regex, num, re.IGNORECASE)
+
+
+class TypeBool(_Type):
+    pass
+
+
+class TypeList(_Type):
+    def getitem(self, i):
+        try:
+            return self.__getitem__(i)
+        except IndexError, e:
+            return None
+
 def define_language():
     """
     Defines the pywurfl query language.
@@ -192,61 +279,78 @@ def define_language():
     @return: The definition of the pywurfl query language.
     """
 
+    # Data types to bind to python objects
+    integer = Word(nums).setParseAction(_toNum)
+    boolean = (CaselessKeyword("true") | CaselessKeyword("false")).setParseAction(_toBool)
+    string = (QuotedString("'") | QuotedString('"')).setParseAction(_toStr)
+    types = (integer | boolean | string)('value')
+
+    capability = Word(alphas, alphanums + '_')('capability')
+
     # Select statement
     select_token = CaselessKeyword("select")
     ua_token = CaselessKeyword("ua")
     id_token = CaselessKeyword("id")
     device_token = CaselessKeyword("device")
-    select_type = (device_token | ua_token | id_token).setResultsName("type")
+    select_type = (device_token | ua_token | id_token)("type")
     select_clause = select_token + select_type
-    select_statement = Group(select_clause).setResultsName("select")
+    select_statement = Group(select_clause)("select")
 
-    capability = Word(alphas, alphanums + '_').setResultsName("capability")
-    integer = Word(nums)
-    boolean = CaselessKeyword("true") | CaselessKeyword("false")
-    string = QuotedString("'") | QuotedString('"')
-    value = (integer | boolean | string).setResultsName("value")
-    binop = oneOf("= != < > >= <=", caseless=True).setResultsName("operator")
+    expr = Forward()
+
+    # class methods
+    method_arg = (types | Group(expr))
+    method_args = Group(ZeroOrMore(delimitedList(method_arg)))('method_args')
+
+    # class attribute
+    attribute = Word(alphas + '_', alphanums + '_')("attribute")
+    attribute_call = (attribute + Suppress('(') + method_args +
+                      Suppress(')'))("attribute_call")
+    # To support method and attribute list like .lower().upper()
+    attribute_concat = Group(ZeroOrMore(Group(Suppress('.') + (attribute_call | attribute))))('attribute_concat')
+
+    expr << Group(types + attribute_concat | capability + attribute_concat)('expr')
+
+    binop = oneOf("= != < > >= <=", caseless=True)("operator")
     and_ = CaselessKeyword("and")
     or_ = CaselessKeyword("or")
 
+    expr_list = (expr + ZeroOrMore(Suppress(',') + expr))
+
     # Any test
-    capabilities = (capability + Suppress(',') + capability +
-                    ZeroOrMore(Suppress(',') + capability))
     any_token = CaselessKeyword("any")
-    any_list = capabilities.setResultsName("any_caps")
-    any_test = (any_token + Suppress('(') + any_list + Suppress(')') +
-                     binop + value)
+    any_expr_list = expr_list("any_expr_list")
+    any_statement = (any_token + Suppress('(') + any_expr_list + Suppress(')') +
+                     binop + expr("rexpr"))('any_statement')
+
     # All test
     all_token = CaselessKeyword("all")
-    all_list = capabilities.setResultsName("all_caps")
-    all_test = (all_token + Suppress('(') + all_list + Suppress(')') +
-                     binop + value)
+    all_expr_list = expr_list("all_expr_list")
+    all_statement = (all_token + Suppress('(') + all_expr_list + Suppress(')') +
+                     binop + expr("rexpr"))('all_statement')
+
     # Capability test
-    cap_test = capability + binop + value
+    expr_test = expr('lexpr') + binop + expr('rexpr')
 
     # WHERE statement
-    boolop = (and_ | or_).setResultsName('boolop')
+    boolop = (and_ | or_)('boolop')
     where_token = CaselessKeyword("where")
 
-    where_test = all_test | any_test | cap_test
+    where_test = (all_statement | any_statement | expr_test)('where_test')
     where_expression = Forward()
-    where_expression << (Group(where_test + ZeroOrMore(boolop +
-                                                       where_expression)
-                               ).setResultsName('where'))
+    where_expression << Group(where_test + ZeroOrMore(boolop + where_expression))('where_expression')
 
     #where_expression << (Group(where_test + ZeroOrMore(boolop +
     #                                                   where_expression) +
-    #                           StringEnd()).setResultsName('where'))
+    #                           StringEnd())('where'))
     #where_expression = (Group(where_test + ZeroOrMore(boolop + where_test) +
-    #                    StringEnd()).setResultsName('where'))
+    #                    StringEnd())('where'))
 
     where_statement = where_token + where_expression
 
     # Mon Jan  1 12:35:56 EST 2007
     # If there isn't a concrete end to the string pyparsing will not parse
     # query correctly
-
     return select_statement + where_statement + '*' + StringEnd()
 
 
@@ -264,8 +368,6 @@ def get_operators():
         def and_tester(value):
             """Tests a device by 'anding' the two following functions:"""
             return func1(value) and func2(value)
-        and_tester.__doc__ = '\n'.join([and_tester.__doc__, func1.__doc__,
-                                        func2.__doc__])
         return and_tester
 
     def or_(func1, func2):
@@ -275,8 +377,6 @@ def get_operators():
         def or_tester(value):
             """Tests a device by 'oring' the two following functions:"""
             return func1(value) or func2(value)
-        or_tester.__doc__ = '\n'.join([or_tester.__doc__, func1.__doc__,
-                                       func2.__doc__])
         return or_tester
 
     return {'=':operator.eq, '!=':operator.ne, '<':operator.lt,
@@ -287,31 +387,103 @@ def get_operators():
 ops = get_operators()
 
 
-def capability_test(cap, op, val):
+def expr_test(lexpr, op, rexpr):
     """
-    Returns a capability test function.
+    Returns an exp test function.
 
-    @param cap: A WURFL capability
-    @type cap: string
+    @param lexpr: An expr
+    @type lexpr: expr
     @param op: A binary test operator
     @type op: string
-    @param val: The value to test for
-    @type val: string
+    @param rexpr: An expr
+    @type rexpr: expr
 
     @rtype: function
     """
 
-    try:
-        val = int(val)
-    except ValueError:
-        if val == 'true':
-            val = True
-        elif val == 'false':
-            val = False
-    def capability_tester(devobj):
-        return ops[op](getattr(devobj, cap), val)
-    capability_tester.__doc__ = "Test a device for %s %s %s" % (cap, op, val)
-    return capability_tester
+    def expr_tester(devobj):
+
+        def evaluate(expression):
+            value = None
+            if expression.keys() == ['expr']:
+                expression = expression.expr
+            # check wheather the expression is a capability or not
+            if 'capability' in expression.keys():
+                capability = expression.capability
+                try:
+                    py_value = getattr(devobj, capability)
+                except AttributeError, e:
+                    raise QueryLanguageError("Invalid capability '%s'" %
+                                             capability)
+
+                if isinstance(py_value, bool):
+                    value = TypeBool(py_value)
+                elif isinstance(py_value, int):
+                    value = TypeNum(py_value)
+                elif isinstance(py_value, str):
+                    value = TypeStr(py_value)
+                else:
+                    raise QueryLanguageError("Unknown type '%s'" %
+                                             py_value.__class__)
+            else:
+                value = expression.value
+
+            for attribute in expression.attribute_concat:
+                py_value = None
+                if 'attribute_call' in attribute.keys():
+                    method_name = attribute.attribute_call.attribute
+                    method_args = []
+                    for method_arg in attribute.attribute_call.method_args:
+                        method_arg_value = None
+                        try:
+                            method_arg_value = evaluate(method_arg.expression)
+                        except AttributeError, e:
+                            method_arg_value = method_arg
+
+                        method_args.append(method_arg_value.py_value)
+
+                    try:
+                        attr = getattr(value, method_name)
+                        py_value = attr(*method_args)
+                    except (AttributeError, TypeError), e:
+                        msg = "'%s' object has no callable attribute '%s'"
+                        raise QueryLanguageError(msg %
+                                                 (type(value.py_value).__name__,
+                                                  method_name))
+                elif 'attribute' in attribute.keys():
+                    try:
+                        py_value = getattr(value, attribute.attribute)
+                    except AttributeError, e:
+                        raise QueryLanguageError(str(e))
+                    if callable(py_value):
+                        msg = "'%s' object has no attribute '%s'" 
+                        raise QueryLanguageError(msg %
+                                                 (type(value.py_value).__name__,
+                                                  attribute.attribute))
+                else:
+                    raise QueryLanguageError('query syntax error')
+
+                if isinstance(py_value, bool):
+                    value = TypeBool(py_value)
+                elif py_value is None:
+                    value = TypeNone(py_value)
+                elif isinstance(py_value, int):
+                    value = TypeNum(py_value)
+                elif isinstance(py_value, str):
+                    value = TypeStr(py_value)
+                elif isinstance(py_value, (list, tuple)):
+                    value = TypeList(py_value)
+                else:
+                    raise QueryLanguageError("Unknown type '%s'" %
+                                             py_value.__class__)
+
+            return value
+
+        lvalue = evaluate(lexpr)
+        rvalue = evaluate(rexpr)
+        return ops[op](lvalue.py_value, rvalue.py_value)
+
+    return expr_tester
 
 
 def combine_funcs(funcs):
@@ -381,29 +553,29 @@ def reduce_statement(exp):
     """
 
     funcs = []
-    if exp.any_caps:
-        for cap in exp.any_caps:
-            funcs.append(capability_test(cap, exp.operator, exp.value))
+    if exp.any_statement:
+        for expr in exp.any_statement.any_expr_list:
+            funcs.append(expr_test(expr, exp.operator, exp.rexpr))
         return reduce_funcs(ops['or'], funcs)
-    elif exp.all_caps:
-        for cap in exp.all_caps:
-            funcs.append(capability_test(cap, exp.operator, exp.value))
+    elif exp.all_statement:
+        for expr in exp.all_statement.all_expr_list:
+            funcs.append(expr_test(expr, exp.operator, exp.rexpr))
         return reduce_funcs(ops['and'], funcs)
 
 
 def test_generator(ql_result):
     """
     Produces a function that encapsulates all the tests from a where
-    statement that takes a Device class or object as a parameter::
+    statement and takes a Device class or object as a parameter::
 
         In (a result object from the following query):
           select id where ringtone=true and any(ringtone_mp3, ringtone_awb)=true
 
         Out:
           def func(devobj):
-              if (devobj.ringtone == true and
-                  (devobj.ringtone_mp3 == true or
-                   devobj.ringtone_awb == true)):
+              if (devobj.ringtone == True and
+                  (devobj.ringtone_mp3 == True or
+                   devobj.ringtone_awb == True)):
                   return True
               else:
                   return False
@@ -414,19 +586,20 @@ def test_generator(ql_result):
     """
 
     funcs = []
-    exp = ql_result.where
-    while exp:
-        if exp.any_caps or exp.all_caps:
-            func = reduce_statement(exp)
+    where_test = ql_result.where_expression
+    while where_test:
+        if where_test.any_statement or where_test.all_statement:
+            func = reduce_statement(where_test)
         else:
-            func = capability_test(exp.capability, exp.operator, exp.value)
+            func = expr_test(where_test.lexpr, where_test.operator,
+                             where_test.rexpr)
 
-        boolop = exp.boolop
+        boolop = where_test.boolop
         if boolop:
             funcs.extend([func, boolop])
         else:
             funcs.append(func)
-        exp = exp.where
+        where_test = where_test.where_expression
     return combine_funcs(funcs)
 
 
@@ -454,8 +627,7 @@ def QL(devices):
         @type instance: boolean
         @rtype: generator
         """
-        qstr = qstr.replace('\n', ' ')
-        qstr = qstr + '*'
+        qstr = qstr.replace('\n', ' ').replace('\r', ' ') + '*'
         try:
             qres = language.parseString(qstr)
             tester = test_generator(qres)
@@ -476,3 +648,4 @@ def QL(devices):
             raise QueryLanguageError(str(exception))
     setattr(devices, 'query', query)
     return query
+
