@@ -1,5 +1,5 @@
 # pywurfl - Wireless Universal Resource File Tools in Python
-# Copyright (C) 2006 Armand Lynch
+# Copyright (C) 2006-2008 Armand Lynch
 #
 # This library is free software; you can redistribute it and/or modify it
 # under the terms of the GNU Lesser General Public License as published by the
@@ -22,7 +22,6 @@ __doc__ = \
 pywurfl WURFL class hierarchy serializer
 """
 
-
 from time import ctime
 
 try:
@@ -31,11 +30,11 @@ except ImportError:
     from elementtree.ElementTree import ElementTree, Element, SubElement
 
 from pywurfl import __version__ as pw_version
-from pywurfl.exceptions import BaseException
+from pywurfl.exceptions import WURFLException
 
 
 __author__ = "Armand Lynch <lyncha@users.sourceforge.net>"
-__copyright__ = "Copyright 2006, Armand Lynch"
+__copyright__ = "Copyright 2006-2008, Armand Lynch"
 __license__ = "LGPL"
 __url__ = "http://celljam.net/"
 __all__ = ['Serialize']
@@ -57,7 +56,7 @@ class Serialize(object):
                     return group
             else:
                 msg = "'%s' capability does not belong to a generic group"
-                raise BaseException(msg % cap)
+                raise WURFLException(msg % cap)
         return group
 
     def _get_value(self, x):
