@@ -1,5 +1,5 @@
 # WURFL Processor - Wireless Universal Resource File Processor in Python
-# Copyright (C) 2006 Armand Lynch
+# Copyright (C) 2004-2008 Armand Lynch
 #
 # This library is free software; you can redistribute it and/or modify it
 # under the terms of the GNU Lesser General Public License as published by the
@@ -34,19 +34,19 @@ except ImportError:
     except ImportError:
         from elementtree.ElementTree import parse
 
-from pywurfl.exceptions import BaseException
+from pywurfl.exceptions import WURFLException
 
 
 __author__ = "Armand Lynch <lyncha@users.sourceforge.net>"
 __contributors__ = "Pau Aliagas <pau@newtral.org>"
-__copyright__ = "Copyright 2004-6, Armand Lynch"
+__copyright__ = "Copyright 2004-2008, Armand Lynch"
 __license__ = "LGPL"
 __url__ = "http://celljam.net/"
 __all__ = ['DeferredDeviceError', 'DeviceHandler', 'WurflProcessor', 'main', 
            'op']
 
 
-class DeferredDeviceError(BaseException):
+class DeferredDeviceError(WURFLException):
     """
     Deferred Device Error Exception
 
